/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Arto Jalkanen <ajalkane@gmail.com>
 *              Niklas Wenzel <nikwen.developer@gmail.com>
 */
import QtQuick 2.3
import Ubuntu.Content 1.0
import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 1.0
import Ubuntu.Components.ListItems 1.0
import org.nemomobile.folderlistmodel 1.0
import com.ubuntu.Archives 0.1
import "../components"
import "../content-hub/filters.js" as Filters

Page {
    id: folderListPage
    title: fileSelector.activeTransfer.direction == ContentTransfer.Import ? i18n.tr("Open…") : i18n.tr("Save to…")

    /* Go to last folder visited */
    head.backAction: Action {
        id: back
        objectName: "back"
        iconName: "back"
        enabled: folderListPage.folder != userplaces.locationHome

        onTriggered: {
            goBack()
        }
    }

    head.actions: [
        Action {
            id: createNewFolder
            objectName: "createFolder"
            iconName: "add"
            text: i18n.tr("New Folder")
            enabled: folderListPage.folder != userplaces.locationHome
            visible: fileSelector.activeTransfer.direction != ContentTransfer.Import
            onTriggered: {
                PopupUtils.open(createFolderDialog, folderListPage)
            }
        }
    ]
    flickable: folderIconView.flickable

    property variant fileView: folderListPage
    property string folder
    property bool loading: pageModel.awaitingResults

    property FolderListSelection selectionManager: pageModel.selectionObject()

    onFlickableChanged: {
        if (flickable === null) {
            folderIconView.flickable.topMargin = 0
        } else {
            folderIconView.flickable.topMargin = units.gu(9.5)
        }
    }

    FolderListModel {
        id: pageModel
        path: folderListPage.folder
        enableExternalFSWatcher: true
        onlyAllowedPaths: true
        nameFilters: Filters.filtersForContentType(fileSelector.activeTransfer.contentType)

        Component.onCompleted: {
            // Add default allowed paths
            addAllowedDirectory(userplaces.locationDocuments)
            addAllowedDirectory(userplaces.locationDownloads)
            addAllowedDirectory(userplaces.locationMusic)
            addAllowedDirectory(userplaces.locationPictures)
            addAllowedDirectory(userplaces.locationVideos)
        }
    }

    FolderListModel {
        id: repeaterModel
        path: folderListPage.folder

        onPathChanged: {
            console.log("Path changed to: " + repeaterModel.path)
        }
    }

    Component {
        id: createFolderDialog
        ConfirmDialogWithInput {
            title: i18n.tr("Create folder")
            text: i18n.tr("Enter name for new folder")

            onAccepted: {
                console.log("Create folder accepted", inputText)
                if (inputText !== '') {
                    pageModel.mkdir(inputText)
                } else {
                    console.log("Empty directory name, ignored")
                }
            }
        }
    }

    Item {
        id: bottomBar
        anchors {
            bottom: parent.bottom
            bottomMargin: units.gu(1) 
            left: parent.left
            right: parent.right
        }
        height: bottomBarButtons.visible ? bottomBarButtons.height + units.gu(2) : 0
    
        Flow {
        id: bottomBarButtons
        anchors.centerIn: parent

        spacing: units.gu(4)

        Button {
            text: fileSelector.activeTransfer.direction == ContentTransfer.Import ? i18n.tr("Open") : i18n.tr("Save")
            enabled: fileSelector.activeTransfer.direction == ContentTransfer.Import && selectionManager.counter > 0 || fileSelector.activeTransfer.direction != ContentTransfer.Import && folderListPage.folder != userplaces.locationHome
            onClicked: {
                if (fileSelector.activeTransfer.direction == ContentTransfer.Import) {
                    var selectedAbsPaths = selectionManager.selectedAbsFilePaths();
                    // For now support only selection in filesystem
                    var selectedAbsUrls = selectedAbsPaths.map(function(item) {
                        return "file://" + item;
                    });
                    console.log("FileSelector OK clicked, selected items: " + selectedAbsUrls);

                    acceptFileSelector(selectedAbsUrls);
                } else {
                    for (var i = 0; i < fileSelector.activeTransfer.items.length; i++) {
                        var filename = fileSelector.activeTransfer.items[i].url.toString().split("/").pop();
                        var j = 1;
                        var origFilename = filename;
                        while (pageModel.existsFile(filename)) {
                            if (origFilename.indexOf(".") > -1) {
                                var ext = origFilename.split(".").pop();
                                filename = origFilename.substr(0, origFilename.length - ext.length) + j.toString() + "." + ext
                            } else {
                                filename = origFilename + j.toString();
                            }
                            j++;
                        }
                        fileSelector.activeTransfer.items[i].move(folderListPage.folder, filename);
                    }
                    fileSelector.activeTransfer.state = ContentTransfer.Collected;
                    Qt.quit();
                }
            }
        }
        Button {
            text: i18n.tr("Cancel")
            onClicked: {
                    console.log("FileSelector cancelled")
                    cancelFileSelector()
                }
            }
        }
    }

    FolderListView {
        id: folderIconView

        clip: true

        folderListModel: pageModel
        anchors {
            top: parent.top
            bottom: bottomBar.top
            left: parent.left
            right: parent.right
        }
    }

    function getArchiveType(fileName) {
        var splitName = fileName.split(".")

        if (splitName.length <= 1) { // To sort out files simply named "zip" or "tar"
            return ""
        }

        var fileExtension = splitName[splitName.length - 1]
        if (fileExtension === "zip") {
            return "zip"
        } else if (fileExtension === "tar") {
            return "tar"
        } else if (fileExtension === "gz") {
            if (splitName.length > 2 && splitName[splitName.length - 2] === "tar") {
                return "tar.gz"
            } else {
                return ""
            }
        } else if (fileExtension === "bz2") {
            if (splitName.length > 2 && splitName[splitName.length - 2] === "tar") {
                return "tar.bz2"
            } else {
                return ""
            }
        } else {
            return ""
        }
    }

    Item {
        id: contents

        anchors {
            top: parent.top
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }

        Label {
            text: i18n.tr("No files")
            fontSize: "large"
            opacity: 0.5
            anchors.centerIn: parent
            visible: folderIconView.count == 0 && !pageModel.awaitingResults
        }

        ActivityIndicator {
            running: pageModel.awaitingResults
            anchors.centerIn: parent
        }
    }

    // Errors from model
    Connections {
        target: pageModel
        onError: {
            console.log("FolderListModel Error Title/Description", errorTitle, errorMessage)
            error(i18n.tr("File operation error"), errorTitle + ": " + errorMessage)
        }
    }

    function goTo(location) {
        // This allows us to enter "~" as a shortcut to the home folder
        // when entering a location on the Go To dialog
        folderListPage.folder = location.replace("~", userplaces.locationHome)
        refresh()
    }

    /* Go to last folder visited */
    function goBack() {
        pageModel.goBack()
        folder = pageModel.path
    }

    /* Go up one directory */
    function goUp() {
        goTo(pageModel.parentPath)
    }

    function refresh() {
        pageModel.refresh()
    }

    function pathAccessedDate() {
        console.log("calling method pageModel.curPathAccessedDate()")
        return pageModel.curPathAccessedDate()
    }

    function pathModifiedDate() {
        console.log("calling method pageModel.curPathModifiedDate()")
        return pageModel.curPathModifiedDate()
    }

    function pathIsWritable() {
        console.log("calling method pageModel.curPathIsWritable()")
        return pageModel.curPathIsWritable()
    }

    // FIXME: hard coded path for icon, assumes Ubuntu desktop icon available.
    // Nemo mobile has icon provider. Have to figure out what's the proper way
    // to get "system wide" icons in Ubuntu Touch, or if we have to use
    // icons packaged into the application. Both folder and individual
    // files will need an icon.
    // TODO: Remove isDir parameter and use new model functions
    function fileIcon(file, isDir) {
        var iconPath = isDir ? "/usr/share/icons/Humanity/places/48/folder.svg"
                             : "/usr/share/icons/Humanity/mimes/48/empty.svg"

        if (file === userplaces.locationHome) {
            iconPath = "../icons/folder-home.svg"
        } else if (file === i18n.tr("~/Desktop")) {
            iconPath = "/usr/share/icons/Humanity/places/48/user-desktop.svg"
        } else if (file === userplaces.locationDocuments) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-documents.svg"
        } else if (file === userplaces.locationDownloads) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-downloads.svg"
        } else if (file === userplaces.locationMusic) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-music.svg"
        } else if (file === userplaces.locationPictures) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-pictures.svg"
        } else if (file === i18n.tr("~/Public")) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-publicshare.svg"
        } else if (file === i18n.tr("~/Programs")) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-system.svg"
        } else if (file === i18n.tr("~/Templates")) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-templates.svg"
        } else if (file === userplaces.locationVideos) {
            iconPath = "/usr/share/icons/Humanity/places/48/folder-videos.svg"
        } else if (file === "/") {
            iconPath = "/usr/share/icons/Humanity/devices/48/drive-harddisk.svg"
        } else if (userplaces.isUserMountDirectory(file)) {
            // In context of Ubuntu Touch this means SDCard currently.
            iconPath = "/usr/share/icons/Humanity/devices/48/drive-removable-media.svg"
        }

        return Qt.resolvedUrl(iconPath)
    }

    function folderDisplayName(folder) {
        if (folder === userplaces.locationHome) {
            return i18n.tr("Home")
        } else if (folder === "/") {
            return i18n.tr("Device")
        } else {
            return basename(folder)
        }
    }

    /* Return depth of current path */
    function pathModel(path){
        if (path === "/") { return 0 } // Otherwise it will return 1
        return path.split("/").length - 1
    }

    /* Return folder name by its depth in current path */
    function pathText(path,index) {
        return basename(path.split('/').slice(0,index+2).join("/"))
    }

    /* Return folder path by its depth in current path */
    function pathRaw(path,index) {
        return path.split('/').slice(0,index+2).join("/")
    }

    function pathName(folder) {
        if (folder === "/") {
            return "/"
        } else {
            return basename(folder)
        }
    }

    function basename(folder) {
        // Returns the latest component (folder) of an absolute path
        // E.g. basename('/home/phablet/Música') returns 'Música'

        // Remove the last trailing '/' if there is one

        folder.replace(/\/$/, "")
        return folder.substr(folder.lastIndexOf('/') + 1)
    }

    function pathExists(path) {
        path = path.replace("~", pageModel.homePath())

        if (path === '/')
            return true

        if(path.charAt(0) === '/') {
            console.log("Directory: " + path.substring(0, path.lastIndexOf('/')+1))
            repeaterModel.path = path.substring(0, path.lastIndexOf('/')+1)
            console.log("Sub dir: " + path.substring(path.lastIndexOf('/')+1))
            if (path.substring(path.lastIndexOf('/')+1) !== "" && !repeaterModel.cdIntoPath(path.substring(path.lastIndexOf('/')+1))) {
                return false
            } else {
                return true
            }
        } else {
            return false
        }
    }

    function itemClicked(model) {
        if (model.isDir) {
            if (model.isReadable && model.isExecutable) {
                console.log("Changing to dir", model.filePath)
                goTo(model.filePath)
            } else {
                PopupUtils.open(Qt.resolvedUrl("NotifyDialog.qml"), delegate,
                                {
                                    title: i18n.tr("Folder not accessible"),
                                    // TRANSLATORS: this refers to a folder name
                                    text: i18n.tr("Can not access %1").arg(model.fileName)

                                })
            }
        } else {
            console.log("Non dir clicked")
            if (fileSelector.activeTransfer.direction == ContentTransfer.Import) {
                if (fileSelector.activeTransfer.selectionType == ContentTransfer.Single) {
                    selectionManager.clear();
                }
                selectionManager.select(model.index, false, true);
            }
        }
    }

    function keyPressed(key, modifiers) {
        if (key === Qt.Key_L && modifiers & Qt.ControlModifier) {
            PopupUtils.open(Qt.resolvedUrl("GoToDialog.qml"), mainView);
            return true;
        } else if (key === Qt.Key_Backspace) {
            goUp()
        }

        return false;
    }

    Component.onCompleted: {
        forceActiveFocus()
    }
}
